# Generated by Django 2.0.7 on 2018-07-23 17:53

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("plans", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Payment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("variant", models.CharField(max_length=255)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("waiting", "Waiting for confirmation"),
                            ("preauth", "Pre-authorized"),
                            ("confirmed", "Confirmed"),
                            ("rejected", "Rejected"),
                            ("refunded", "Refunded"),
                            ("error", "Error"),
                            ("input", "Input"),
                        ],
                        default="waiting",
                        max_length=10,
                    ),
                ),
                (
                    "fraud_status",
                    models.CharField(
                        choices=[
                            ("unknown", "Unknown"),
                            ("accept", "Passed"),
                            ("reject", "Rejected"),
                            ("review", "Review"),
                        ],
                        default="unknown",
                        max_length=10,
                        verbose_name="fraud check",
                    ),
                ),
                ("fraud_message", models.TextField(blank=True, default="")),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("modified", models.DateTimeField(auto_now=True)),
                ("transaction_id", models.CharField(blank=True, max_length=255)),
                ("currency", models.CharField(max_length=10)),
                (
                    "total",
                    models.DecimalField(decimal_places=2, default="0.0", max_digits=9),
                ),
                (
                    "delivery",
                    models.DecimalField(decimal_places=2, default="0.0", max_digits=9),
                ),
                (
                    "tax",
                    models.DecimalField(decimal_places=2, default="0.0", max_digits=9),
                ),
                ("description", models.TextField(blank=True, default="")),
                ("billing_first_name", models.CharField(blank=True, max_length=256)),
                ("billing_last_name", models.CharField(blank=True, max_length=256)),
                ("billing_address_1", models.CharField(blank=True, max_length=256)),
                ("billing_address_2", models.CharField(blank=True, max_length=256)),
                ("billing_city", models.CharField(blank=True, max_length=256)),
                ("billing_postcode", models.CharField(blank=True, max_length=256)),
                ("billing_country_code", models.CharField(blank=True, max_length=2)),
                ("billing_country_area", models.CharField(blank=True, max_length=256)),
                ("billing_email", models.EmailField(blank=True, max_length=254)),
                (
                    "customer_ip_address",
                    models.GenericIPAddressField(blank=True, null=True),
                ),
                ("extra_data", models.TextField(blank=True, default="")),
                ("message", models.TextField(blank=True, default="")),
                ("token", models.CharField(blank=True, default="", max_length=36)),
                (
                    "captured_amount",
                    models.DecimalField(decimal_places=2, default="0.0", max_digits=9),
                ),
                (
                    "order",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="plans.Order",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import os
import sys

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.settings")
    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



import json
import logging
from decimal import Decimal
from urllib.parse import urljoin

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.db import models
from django.dispatch.dispatcher import receiver
from django.urls import reverse
from payments import PaymentStatus, PurchasedItem
from payments.core import get_base_url
from payments.models import BasePayment
from payments.signals import status_changed
from plans.contrib import get_user_language, send_template_email
from plans.models import Order
from plans.signals import account_automatic_renewal

from .views import create_payment_object

logger = logging.getLogger(__name__)


class Payment(BasePayment):
    order: Order = models.ForeignKey(
        "plans.Order",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    transaction_fee: models.DecimalField = models.DecimalField(
        max_digits=9,
        decimal_places=2,
        default=Decimal("0.0"),
    )
    autorenewed_payment: models.BooleanField = models.BooleanField(
        default=False,
    )

    def clean(self):
        if self.order.status == Order.STATUS.COMPLETED:
            confirmed_payment_count = self.order.payment_set.exclude(pk=self.pk)
            confirmed_payment_count = confirmed_payment_count.filter(
                status=PaymentStatus.CONFIRMED
            ).count()
            if self.status != PaymentStatus.CONFIRMED and confirmed_payment_count == 0:
                raise ValidationError(
                    {
                        "status": "Can't leave confirmed order without any confirmed payment. "
                        "Please change Order first if you still want to perform this change.",
                    },
                )

    def save(self, **kwargs):
        if "payu" in self.variant:
            # TODO: base this on actual payment methods and currency fees on PayU
            # or even better on real PayU info
            self.transaction_fee = self.total * Decimal("0.029") + Decimal("0.05")
        elif hasattr(self, "extra_data") and self.extra_data:
            extra_data = json.loads(self.extra_data)
            if "response" in extra_data:
                transactions = extra_data["response"]["transactions"]
                for transaction in transactions:
                    related_resources = transaction["related_resources"]
                    if len(related_resources) == 1:
                        sale = related_resources[0]["sale"]
                        if "transaction_fee" in sale:
                            self.transaction_fee = Decimal(
                                sale["transaction_fee"]["value"]
                            )
                        else:
                            logger.warning(
                                "Payment fee not included",
                                extra={
                                    "extra_data": extra_data,
                                },
                            )
        ret_val = super().save(**kwargs)
        return ret_val

    def get_failure_url(self):
        return reverse("order_payment_failure", kwargs={"pk": self.order.pk})

    def get_success_url(self):
        return reverse("order_payment_success", kwargs={"pk": self.order.pk})

    def get_payment_url(self):
        return reverse("payment_details", kwargs={"payment_id": self.pk})

    def get_purchased_items(self):
        yield PurchasedItem(
            name=self.description,
            sku=self.order.pk,
            quantity=1,
            price=self.order.amount,
            tax_rate=(1 + self.order.tax / 100) if self.order.tax else 1,
            currency=self.currency,
        )

    def get_renew_token(self):
        """
        Get the recurring payments renew token for user of this payment
        Used by PayU provider for now
        """
        try:
            recurring_plan = self.order.user.userplan.recurring
            if (
                recurring_plan.token_verified
                and self.variant == recurring_plan.payment_provider
            ):
                return recurring_plan.token
        except ObjectDoesNotExist:
            pass
        return None

    def set_renew_token(
        self,
        token,
        card_expire_year=None,
        card_expire_month=None,
        card_masked_number=None,
        automatic_renewal=True,
    ):
        """
        Store the recurring payments renew token for user of this payment
        The renew token is string defined by the provider
        Used by PayU provider for now
        """
        self.order.user.userplan.set_plan_renewal(
            order=self.order,
            token=token,
            payment_provider=self.variant,
            card_expire_year=card_expire_year,
            card_expire_month=card_expire_month,
            card_masked_number=card_masked_number,
            has_automatic_renewal=automatic_renewal,
        )


@receiver(status_changed)
def change_payment_status(sender, *args, **kwargs):
    payment = kwargs["instance"]
    order = payment.order
    if payment.status == PaymentStatus.CONFIRMED:
        if hasattr(order.user.userplan, "recurring"):
            order.user.userplan.recurring.token_verified = True
            order.user.userplan.recurring.save()
        order.complete_order()
    if order.status != Order.STATUS.COMPLETED and payment.status not in (
        PaymentStatus.CONFIRMED,
        PaymentStatus.WAITING,
        PaymentStatus.INPUT,
    ):
        order.status = Order.STATUS.CANCELED
        order.save()
        if hasattr(order.user.userplan, "recurring"):
            order.user.userplan.recurring.token_verified = False
            order.user.userplan.recurring.save()


@receiver(account_automatic_renewal)
def renew_accounts(sender, user, *args, **kwargs):
    userplan = user.userplan
    if (
        userplan.recurring.payment_provider in settings.PAYMENT_VARIANTS
        and userplan.recurring.has_automatic_renewal
    ):
        order = userplan.recurring.create_renew_order()

        payment = create_payment_object(
            userplan.recurring.payment_provider, order, autorenewed_payment=True
        )

        try:
            redirect_url = payment.auto_complete_recurring()
        except Exception as e:
            print(f"Exceptin during automatic renewal: {e}")
            logger.exception(
                "Exception during account renewal",
                extra={
                    "payment": payment,
                },
            )
            redirect_url = urljoin(
                get_base_url(),
                reverse(
                    "create_order_plan", kwargs={"pk": order.get_plan_pricing().pk}
                ),
            )

        if redirect_url != "success":
            print("CVV2/3DS code is required, enter it at %s" % redirect_url)
            send_template_email(
                [payment.order.user.email],
                "mail/renew_cvv_3ds_title.txt",
                "mail/renew_cvv_3ds_body.txt",
                {"redirect_url": redirect_url},
                get_user_language(payment.order.user),
            )
        if payment.status == PaymentStatus.CONFIRMED:
            order.complete_order()



from django import template
from django.conf import settings

register = template.Library()


# settings value
@register.inclusion_tag("plans_payments/payment_buttons.html")
def payment_buttons(object_variable):
    variants = getattr(settings, "PAYMENT_VARIANTS", [])
    return {
        "variants": variants,
        "object": object_variable,
    }



from decimal import Decimal

from django.contrib.auth.mixins import LoginRequiredMixin
from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse
from django.urls import reverse, reverse_lazy
from django.views.generic import View
from payments import RedirectNeeded, get_payment_model
from plans.models import Order


class PaymentDetailView(LoginRequiredMixin, View):
    login_url = reverse_lazy("auth_login")
    template_name = "plans_payments/payment.html"

    def get(self, request, *args, payment_id=None):
        payment = get_object_or_404(
            get_payment_model(), order__user=request.user, id=payment_id
        )
        try:
            form = payment.get_form(data=request.POST or None)
        except RedirectNeeded as redirect_to:
            payment.save()
            return redirect(str(redirect_to))
        return TemplateResponse(
            request, "plans_payments/payment.html", {"form": form, "payment": payment}
        )


def get_client_ip(request):
    return request.META.get("REMOTE_ADDR")


def create_payment_object(
    payment_variant, order, request=None, autorenewed_payment=False
):
    Payment = get_payment_model()
    if (
        hasattr(order.user.userplan, "recurring")
        and order.user.userplan.recurring.payment_provider != payment_variant
    ):
        order.user.userplan.recurring.delete()
    return Payment.objects.create(
        variant=payment_variant,
        order=order,
        description="Subscription plan %s purchase" % order.name,
        total=Decimal(order.total()),
        tax=Decimal(order.tax_total()),
        currency=order.currency,
        delivery=Decimal(0),
        billing_first_name=order.user.first_name,
        billing_last_name=order.user.last_name,
        billing_email=order.user.email or "",
        billing_address_1=order.user.billinginfo.street,
        # billing_address_2=order.user.billinginfo.zipcode,
        billing_city=order.user.billinginfo.city,
        billing_postcode=order.user.billinginfo.zipcode,
        billing_country_code=order.user.billinginfo.country,
        # billing_country_area=order.user.billinginfo.zipcode,
        customer_ip_address=get_client_ip(request) if request else "127.0.0.1",
        autorenewed_payment=autorenewed_payment,
    )


class CreatePaymentView(LoginRequiredMixin, View):
    login_url = reverse_lazy("auth_login")

    def get(self, request, *args, order_id=None, payment_variant=None):
        order = get_object_or_404(Order, pk=order_id, user=request.user)
        payment = create_payment_object(payment_variant, order, request)
        return redirect(reverse("payment_details", kwargs={"payment_id": payment.id}))



# Generated by Django 2.0.9 on 2018-11-04 19:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("plans_payments", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="payment",
            name="transaction_fee",
            field=models.DecimalField(decimal_places=2, default="0.0", max_digits=9),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.contrib.auth import views as auth_views
from django.urls import include, path

urlpatterns = [
    path("", include("plans_payments.urls")),
    path("login/", auth_views.LoginView.as_view(), name="auth_login"),
    path("", include("plans.urls")),
]



from django.contrib import admin
from django.contrib.admin import SimpleListFilter
from payments import PaymentStatus
from plans.models import Order
from related_admin import RelatedFieldAdmin

from . import models


class FaultyPaymentsFilter(SimpleListFilter):
    title = "faulty_payments"
    parameter_name = "faulty_payments"

    def lookups(self, request, model_admin):
        return [
            ("unconfirmed_order", "Confirmed payment unconfirmed order"),
        ]

    def queryset(self, request, queryset):
        if self.value() == "unconfirmed_order":
            return queryset.filter(status=PaymentStatus.CONFIRMED).exclude(
                order__status=Order.STATUS.COMPLETED
            )
        return queryset


@admin.register(models.Payment)
class PaymentAdmin(RelatedFieldAdmin):
    list_display = (
        "id",
        "transaction_id",
        "token",
        "order__user",
        "variant",
        "status",
        "fraud_status",
        "currency",
        "total",
        "customer_ip_address",
        "tax",
        "transaction_fee",
        "captured_amount",
        "created",
        "modified",
        "autorenewed_payment",
    )
    list_filter = (
        "status",
        "variant",
        "fraud_status",
        "currency",
        "autorenewed_payment",
        FaultyPaymentsFilter,
    )
    search_fields = (
        "order__user__first_name",
        "order__user__last_name",
        "order__user__email",
        "transaction_id",
        "extra_data",
        "token",
    )
    list_select_related = ("order__user",)
    autocomplete_fields = ("order",)
    readonly_fields = (
        "created",
        "modified",
    )



from django.urls import path

from .views import CreatePaymentView, PaymentDetailView

urlpatterns = [
    path(
        "payment_details/<int:payment_id>/",
        PaymentDetailView.as_view(),
        name="payment_details",
    ),
    path(
        "create_payment/<str:payment_variant>/<int:order_id>/",
        CreatePaymentView.as_view(),
        name="create_payment",
    ),
]
